//  Demo 11: FirstMovingSquareThread.java
//
//     A Simple Example of a Java Thread for Multi-Tasking.
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999
//
// A FirstMovingSquareThread object animates a Square object in a 
// separate Thread of execution.  In the animation sequence, the
// Square moves across the screen.

// You may have noticed that it is difficult to stop the 
// previous demos while the animation is continuing.  The reason
// is that the application cannot respond to events while the
// for loop in paint is continuing.

// By running the animation in a separate Thread, we can avoid this
// problem.  What are Threads anyway?  You are familiar with the
// fact that the machine you work on appears to do several things 
// at once -- for example displaying a new Window while a program 
// continues to run in an old Window.  Actually, the machine only 
// executes one instruction at a time, but it achieves the ILLUSION 
// of concurrency by running the instructions in one process for a 
// few milliseconds, say your new Window, and then switching to the
// the instructions in another process for another few milliseconds,
// say the program in your old Window.  This is called "time slicing"
// because the processor time is "sliced up" between separate 
// processes.

// A Thread is another name for a process.  (Some will disagree with 
// this and assert that a Thread is a "lightweight process", but this
// need not concern us here.)  The name "Thread" comes from the 
// phrase, "separate Thread of execution".  A Java program composed 
// of several Threads appears to do several things at once, even 
// though it really does only one thing at a time -- at least on a 
// single processor machine, the kind most of you have.  Such a Java
// program appears to do several things at once by the same time
// slicing mechanism described earlier.

// Fortunately, Java and the operating system hide the ugly time 
// slicing details, so we can concentrate on making our multi-
// threaded program work.  This demo and the next illustrate a very 
// simple use of Threads by making the animation run in a separate 
// Thread of execution.  This way, the main program (the application)
// can respond to events while the animation is running.

// By the way, the words "concurrency", "multi-tasking" and "multi-
// threading" all mean the same thing.  The title of this course
// includes the word "multi-tasking".

import java.awt.*;

import Square;

// A FirstMovingSquareThread object should be
//
//   1. A Thread so it can run as a separate Thread of execution.
//
// A FirstMovingSquareThread object should know
//
//   1. The Square object it is going to animate (state variable).
//   2. Its parent object, eg. the Frame or Applet Panel on which 
//        the animation will be displayed (state variable).
//   3. The vector direction in which the Square is going to move 
//        (state variables).
//   4. The distance the Square is going to move (state variable).
//   5. How to set and return the Square object (methods).
//   6. How to set and return the parent object (methods).
//   7. How to set and return the vector direction (methods).
//   8. How to set and return the distance (methods).
//   9. How to run itself (method -- all Threads have to have a
//        run method).  Here is where the animation will actually
//        be done.
//
// A FirstMovingSquareThread object
//
//   1. Is NOT a stand alone application, so it does not need a
//      main().
//   2. Is designed to be used by other classes.

public class FirstMovingSquareThread extends Thread {

   private Square s;    // the Square to be animated, to be moved
                        // diagonally across the screen.

   private Component parent;  // the Canvas, Frame or Applet Panel
                              // on which we draw the animation

   private Graphics g;  // our parent's Graphics object, on which
                        // we draw

   private int rx, ry;  // the x and y components of the vector 
                        // (direction) the Square is to move
 
   private int d;       // the distance, in number of vector lengths,
                        // the Square is to move

   private Graphics gBuffer;   // for double buffering, as usual
   private Image imageBuffer;

   // Constructor: as usual, just initializes variables.  The
   // parameters are:
     //  Square s   : the Square to be animated
     //  Component parent : the Canvas, Frame or Applet Panel
     //                     on which we draw the animation
     //  int rx, ry : the x and y components of the vector
     //               (direction) in which the Square will move
     //  int d      : the number of steps to move
   public FirstMovingSquareThread(Square s, Component parent, 
                             int rx, int ry, int d) {

       this.s = s;
       this.parent = parent;
       this.rx = rx;
       this.ry = ry;
       this.d = d;

   } // Constructor FirstMovingSquareThread

   // init(): creates the Graphics buffer object, and initializes
   //         the Graphics object we draw on.  Remember, we can't
   //         do this in our Constructor, because we actually have
   //         to exist before the createImage method will work.
   //         This is an example of an "internal method".  It will
   //         be called from run().  Therefore, it is declared to 
   //         be private.
   private void init() {

     // initialize g to the Graphics object (context) of our parent.
     // g is now the g of our parent's paint(Graphics g) method.
     g = parent.getGraphics();

     // create the gBuffer Graphics buffer object
     //   two step process, more Java obscurity
     imageBuffer = parent.createImage(400, 400);  // 400 by 400 
                                                  // pixels

     // we have to wait while the system creates the imageBuffer,
     // or we will simply get a null pointer.
     while(imageBuffer == null) {

        try {
          Thread.sleep(50);  // 50 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }
     }

     // and finally, we can get our graphics buffer!
     gBuffer = imageBuffer.getGraphics();

     // no need to call repaint() because we are not going to
     // do our drawing in paint()!

   } // init()

   // set our Square object
   public void setSquare(Square s) {

      this.s = s;

   } // setSquare()

   // return our Square object
   public Square getSquare() {

      return s;

   } // getSquare()

   // set our parent object
   public void setParent(Component parent) {

      this.parent = parent;

   } // setParent()

   // return our Parent object
   public Component getParent() {

      return parent;

   } // getParent()

   // set our Graphics object
   public void setGraphics(Graphics g) {

      this.g = g;

   } // setGraphics()

   // return out Graphics object
   public Graphics getGraphics() {

      return g;

   } // getGraphics()

   // set our vector x component
   public void setRx(int rx) {

      this.rx = rx;

   } // setRx()

   // return our vector x component
   public int getRx() {

      return rx;

   } // getRx()

   // set our vector y component
   public void setRy(int ry) {

      this.ry = ry;

   } // setRy()

   // return our vector y component
   public int getRy() {

      return ry;

   } // getRy()

   // set our distance
   public void setDistance(int d) {

      this.d = d;

   } // setDistance()

   // return our distance
   public int getDistance() {

      return d;

   } // getDistance()

   // Here is where we do our assigned task, animate the Square!
   // All Threads have to have a run method.
   public void run() {

      Color bg = parent.getBackground();  // background color

      init();  // initialize Graphics and Image buffers

      // This is interesting.  For the first time, we are going to
      // do our drawing outside of the paint() method.  This is so
      // we can continually draw in one Thread of execution while
      // our creator application (a Frame or an Applet) does other
      // things like event processing.  Also, we don't have a 
      // paint() method because we are not a Frame or Applet Panel.

      // Here comes the animation!
      for(int i = 1; i <= d; i++) {

         // move -- do drawing offscreen on the Graphics
         //         buffer object
         s.draw(gBuffer, bg);  // hide square
         s.move(rx, ry);
         s.draw(gBuffer);

         //draw completed image on the screen
         g.drawImage(imageBuffer, 0, 0, parent);

         // To pause, so the animation doesn't go too fast, use
         // the sleep() method of the Thread class.  Unfortunately,
         // it throws an InterruptedException, so we have to embed
         // our simple pause statement in a try -- catch clause.
         try {
           Thread.sleep(20);  // 20 ms pause
         }
         catch (InterruptedException e) {
           // do nothing
         }
       }

   } // run()

} // class FirstMovingSquareThread

   